#import <Foundation/Foundation.h>
#import <OmniBase/OmniBase.h>
#import <OmniFoundation/OmniFoundation.h>
#import <OmniAIAT/OmniAIAT.h>

void usage(const char *name);
void search(NSString *indexName, NSString *queryExpression);
void createIndex(NSString *indexName, NSString *directoryPath);
void updateIndex(NSString *indexName, NSString *directoryPath);

typedef enum _OIndexAction {
    OIndexUnknownAction = 0,
    OIndexCreateIndexAction,
    OIndexUpdateIndexAction,
    OIndexSearchIndexAction
} OIndexAction;

int main(int argc, const char *argv[])
{
    NSAutoreleasePool *pool;
    NSUserDefaults *userDefaults;
    NSString *indexName;
    NSString *directoryPath = nil;
    NSString *queryExpression = nil;
    OIndexAction action = OIndexUnknownAction;

    pool = [[NSAutoreleasePool alloc] init];
    [OBPostLoader processClasses];

    userDefaults = [NSUserDefaults standardUserDefaults];
    if ((indexName = [userDefaults stringForKey:@"create"])) {
        action = OIndexCreateIndexAction;
        if (!(directoryPath = [userDefaults stringForKey:@"dir"]))
            usage(argv[0]);
    } else if ((indexName = [userDefaults stringForKey:@"update"])) {
        action = OIndexUpdateIndexAction;
        if (!(directoryPath = [userDefaults stringForKey:@"dir"]))
            usage(argv[0]);
    } else if ((indexName = [userDefaults stringForKey:@"search"])) {
        action = OIndexSearchIndexAction;
        if (!(queryExpression = [userDefaults stringForKey:@"query"]))
            usage(argv[0]);
    } else
        usage(argv[0]);

    switch (action) {
        case OIndexCreateIndexAction:
            createIndex(indexName, directoryPath);
            break;
        case OIndexUpdateIndexAction:
            updateIndex(indexName, directoryPath);
            break;
        case OIndexSearchIndexAction:
            search(indexName, queryExpression);
            break;
        default:
            usage(argv[0]);
    }

    [pool release];
    return 0;
}

void usage(const char *name)
{
    fprintf(stderr, "%s usage:\n", name);
    fprintf(stderr, "     %s -search index -query query [-max hits (default 25)]\n", name);
    fprintf(stderr, "     %s -create index -dir directory\n", name);
    fprintf(stderr, "     %s -update index -dir directory\n", name);
    exit(-1);
}

void search(NSString *indexName, NSString *queryExpression)
{
    NSUserDefaults *userDefaults;
    OAIIndex *index;
    OAIQuery *query;
    unsigned int maxHits;
    unsigned int resultIndex, resultCount;

    index = [[[OAIIndex alloc] initWithIndex:indexName
                                 forDirectory:nil] autorelease];
    userDefaults = [NSUserDefaults standardUserDefaults];
    if (!(maxHits = [userDefaults integerForKey:@"max"]))
        maxHits = 25;
    query = [[[OAIQuery alloc] initWithQuery:queryExpression
                                   onIndices:[NSArray arrayWithObject:index]
                                     maxHits:maxHits] autorelease];

    for (resultCount = [query resultCount], resultIndex = 0;
         resultIndex < resultCount; resultIndex++) {
        float score;

        score = [query scoreAtIndex:resultIndex];
        printf("%2.2f    %s\n", score, [[query pathAtIndex:resultIndex] cString]);
    }
}

void createIndex(NSString *indexName, NSString *directoryPath)
{
    OAITokenizerRegistry *registry;
    OAIIndex *index;

    //  We should support a series of flags to specify a mapping between extensions
    //  and tokenizers (and where to find them), so we can set those up in the
    //  tokenizer registry.
    //
    //registry = [OAITokenizerRegistry sharedRegistry];
    //tokenizer = [[[OMP3Tokenizer alloc] init] autorelease];
    //[registry registerTokenizer:tokenizer forExtension:@"mp3"];

    registry = [OAITokenizerRegistry sharedRegistry];
    [registry registerDefaultTokenizers];

    index = [[[OAIIndex alloc] initNewIndex:indexName
                               forDirectory:directoryPath] autorelease];
    
    [index update];
}

void updateIndex(NSString *indexName, NSString *directoryPath)
{
    OAITokenizerRegistry *registry;
    OAIIndex *index;

    //  We should support a series of flags to specify a mapping between extensions
    //  and tokenizers (and where to find them), so we can set those up in the
    //  tokenizer registry.
    //
    //registry = [OAITokenizerRegistry sharedRegistry];
    //tokenizer = [[[OMP3Tokenizer alloc] init] autorelease];
    //[registry registerTokenizer:tokenizer forExtension:@"mp3"];

    registry = [OAITokenizerRegistry sharedRegistry];
    [registry registerDefaultTokenizers];

    index = [[[OAIIndex alloc] initWithIndex:indexName
                                forDirectory:directoryPath] autorelease];
    [index update];
}
